@@ -50,7 +50,7 @@ class CallbackResult:
 class RequestMatch(object):
     url_or_pattern = None  # type: Union[URL, Pattern]
 
-    def __init__(self, url: Union[str, Pattern],
+    def __init__(self, url: Union[URL, str, Pattern],
                  method: str = hdrs.METH_GET,
                  status: int = 200,
                  body: str = '',
@@ -139,6 +139,8 @@ class RequestMatch(object):
             kwargs['traces'] = []
             kwargs['loop'] = loop
             kwargs['session'] = None
+        else:
+            loop = None
         # We need to initialize headers manually
         _headers = CIMultiDict({hdrs.CONTENT_TYPE: content_type})
         if headers:
@@ -154,7 +156,7 @@ class RequestMatch(object):
             resp.raw_headers = raw_headers
         resp.status = status
         resp.reason = reason
-        resp.content = stream_reader_factory()
+        resp.content = stream_reader_factory(loop)
         resp.content.feed_data(body)
         resp.content.feed_eof()
         return resp
@@ -165,7 +167,10 @@ class RequestMatch(object):
         if isinstance(self.exception, Exception):
             return self.exception
         if callable(self.callback):
-            result = self.callback(url, **kwargs)
+            if asyncio.iscoroutinefunction(self.callback):
+                result = await self.callback(url, **kwargs)
+            else:
+                result = self.callback(url, **kwargs)
         else:
             result = None
         result = self if result is None else result
@@ -244,28 +249,28 @@ class aioresponses(object):
         self.patcher.stop()
         self.clear()
 
-    def head(self, url: 'Union[URL, str]', **kwargs):
+    def head(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_HEAD, **kwargs)
 
-    def get(self, url: 'Union[URL, str]', **kwargs):
+    def get(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_GET, **kwargs)
 
-    def post(self, url: 'Union[URL, str]', **kwargs):
+    def post(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_POST, **kwargs)
 
-    def put(self, url: 'Union[URL, str]', **kwargs):
+    def put(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_PUT, **kwargs)
 
-    def patch(self, url: 'Union[URL, str]', **kwargs):
+    def patch(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_PATCH, **kwargs)
 
-    def delete(self, url: 'Union[URL, str]', **kwargs):
+    def delete(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_DELETE, **kwargs)
 
-    def options(self, url: 'Union[URL, str]', **kwargs):
+    def options(self, url: 'Union[URL, str, Pattern]', **kwargs):
         self.add(url, method=hdrs.METH_OPTIONS, **kwargs)
 
-    def add(self, url: 'Union[URL, str]', method: str = hdrs.METH_GET,
+    def add(self, url: 'Union[URL, str, Pattern]', method: str = hdrs.METH_GET,
             status: int = 200,
             body: str = '',
             exception: 'Exception' = None,
@@ -331,4 +336,18 @@ class aioresponses(object):
         key = (method, url)
         self.requests.setdefault(key, [])
         self.requests[key].append(RequestCall(args, kwargs))
+
+        # Automatically call response.raise_for_status() on a request if the
+        # request was initialized with raise_for_status=True. Also call
+        # response.raise_for_status() if the client session was initialized
+        # with raise_for_status=True, unless the request was called with
+        # raise_for_status=False.
+        raise_for_status = kwargs.get('raise_for_status')
+        if raise_for_status is None:
+            raise_for_status = getattr(
+                orig_self, '_raise_for_status', False
+            )
+        if raise_for_status:
+            response.raise_for_status()
+
         return response
